module mine_game(
    input clk,
    input rst,
    /* Keyboard input */
    input ps2_clk,
    input ps2_data,
    /* VGA display */
    output hsync,
    output vsync,
    output [3:0] red,
    output [3:0] green,
    output [3:0] blue,
    /* Seven segment display */
    output [6:0] hundreds,
    output [6:0] tens,
    output [6:0] units,
    /* Dot matrix display */
    output [7:0] dot_row,
    output [7:0] dot_col
);
    /* VGA Clock */
    wire clk_25M;
    /* Matrix Clock */
    wire clk_matrix;
    /* Timer Clock */
    wire clk_1;
    
    /* 
        pointer_x: A unsigned integer indicate the pointer's x position
        pointer_y: A unsigned integer indicate the pointer's y position
        click: 1 bit contain button status.
    */
    wire [3:0] pointer_x;
    wire [3:0] pointer_y;
    wire click;
    
    /* 
        game_map: This variable contains 10x10 blocks' current status.
            |empty|open|bomb|n[2]|n[1]|n[0]|
        block_position: Passing the index of map
        block_data: Return the map data in 'block_position' index.
        game_over: 1 for game_over
    */
    wire [6:0] block_position;
    wire [7:0] block_data;
    wire game_over;
    
    clk_div #(0) u0(clk, clk_25M);
    clk_div #(16667) u1(clk, clk_matrix);
    clk_div #(25000000) u2(clk, clk_1);
    
    keyboard_driver (ps2_clk, ps2_data, rst, pointer_x, pointer_y, click);
    seven_driver (clk_1, rst, game_over, hundreds, tens, units);
    mine_logic (rst, pointer_x, pointer_y, click, block_position, block_data, game_over);
    vga_driver (clk_25M, rst, pointer_x, pointer_y, block_data, block_position, hsync, vsync, red, green, blue);
    matrix_driver (clk_matrix, rst, game_over, dot_row, dot_col);

endmodule
