module vga_driver(
  input clk, 
  input rst,
  input [3:0] x,
  input [3:0] y,
  input [7:0] block_data,
  output [6:0] block_position,
  output hsync,
  output vsync,
  output [3:0] red,
  output [3:0] green,
  output [3:0] blue
);

  wire [9:0] h_counter;
  wire [9:0] v_counter;
  wire [3:0] internal_red;
  wire [3:0] internal_green;
  wire [3:0] internal_blue;

  assign hsync = (h_counter < 92) ? 1'b0 : 1'b1;
  assign vsync = (v_counter < 2) ? 1'b0 : 1'b1;
  assign active = 143 < h_counter && h_counter < 784 && 34 < v_counter && v_counter < 515;
  assign red = active ? internal_red : 4'h0;
  assign green = active ? internal_green : 4'h0;
  assign blue = active ? internal_blue : 4'h0;

  sync_counter (clk, rst, h_counter, v_counter);
  color_control (h_counter, v_counter, x, y, block_data, block_position, internal_red, internal_green, internal_blue);

endmodule
module sync_counter(
  input clk,
  input rst,
  output reg [9:0] h_counter,
  output reg [9:0] v_counter
);

  always @(posedge clk or negedge rst) begin
    if (~rst) begin
      h_counter <= 0;
      v_counter <= 0;
    end else begin  
      if (h_counter <= 799) begin
        h_counter <= h_counter + 1;
      end else begin
        h_counter <= 0;
      end
          
      if (v_counter <= 524) begin
        if (h_counter == 0) begin
          v_counter <= v_counter + 1;
        end else begin
          v_counter <= v_counter;
        end
      end else begin
        v_counter <= 0;
      end
    end
  end

endmodule

module color_control(
  input [9:0] h_counter,
  input [9:0] v_counter,
  input [3:0] x,
  input [3:0] y,
  input [7:0] block_data,
  output [6:0] block_position,
  output reg [3:0] red,
  output reg [3:0] green,
  output reg [3:0] blue
);

  wire [3:0] row_coor, col_coor;
  wire [5:0] i_val, j_val;
  wire [7:0] block_type;

  /* color:
        0: white
        1: black
        2: gray      
  */
  reg [1:0] color;

  assign col_coor = (h_counter - 144) / 64;
  assign row_coor = (v_counter - 35) / 48;
  assign block_position = row_coor * 10 + col_coor;
  assign block_type = block_data & 8'b00111111;
  assign j_val = (h_counter - 144) % 64;
  assign i_val = (v_counter - 35) % 48;

  always @(*) begin
	if(col_coor >= 10 || row_coor >= 10) begin
      color <= 2;
	end else begin
      case(block_type)
        8'b011000: begin	// bomb
          if((0 <= i_val && i_val <= 3) || (44 <= i_val && i_val <= 47) || (0 <= j_val && j_val <= 3) || (60 <= j_val && j_val <= 63)) 
            color <= 1;
          else if(23 <= i_val && i_val <= 26 && 27 <= j_val && j_val <= 38)
            color <= 1;
          else
            color <= 2;
        end
        8'b010001: begin	// 1
          if((0 <= i_val && i_val <= 3) || (44 <= i_val && i_val <= 47) || (0 <= j_val && j_val <= 3) || (60 <= j_val && j_val <= 63)) 
            color <= 1;
          else if(11 <= i_val && i_val <= 38 && 39 <= j_val && j_val <= 42) 
            color <= 1;
          else 
            color <= 2;
        end
        8'b010010: begin	// 2
          if((0 <= i_val && i_val <= 3) || (44 <= i_val && i_val <= 47) || (0 <= j_val && j_val <= 3) || (60 <= j_val && j_val <= 63)) 
            color <= 1;
          else if(7 <= i_val && i_val <= 10 && 27 <= j_val && j_val <= 38)
            color <= 1;
          else if(11 <= i_val && i_val <= 22 && 39 <= j_val && j_val <= 42)
            color <= 1;
          else if(23 <= i_val && i_val <= 26 && 27 <= j_val && j_val <= 38)
            color <= 1;
          else if(27 <= i_val && i_val <= 38 && 23 <= j_val && j_val <= 26)
            color <= 1;
          else if(39 <= i_val && i_val <= 42 && 27 <= j_val && j_val <= 38)
            color <= 1;
          else
            color <= 2;
        end
        8'b010011: begin	// 3
          if((0 <= i_val && i_val <= 3) || (44 <= i_val && i_val <= 47) || (0 <= j_val && j_val <= 3) || (60 <= j_val && j_val <= 63))
            color <= 1;
          else if(7 <= i_val && i_val <= 10 && 27 <= j_val && j_val <= 38)
            color <= 1;
          else if(11 <= i_val && i_val <= 22 && 39 <= j_val && j_val <= 42)
            color <= 1;
          else if(23 <= i_val && i_val <= 26 && 27 <= j_val && j_val <= 38)
            color <= 1;
          else if(27 <= i_val && i_val <= 38 && 39 <= j_val && j_val <= 42)
            color <= 1;
          else if(39 <= i_val && i_val <= 42 && 27 <= j_val && j_val <= 38)
            color <= 1;
          else
            color <= 2;
        end
        8'b010100: begin	// 4
          if((0 <= i_val && i_val <= 3) || (44 <= i_val && i_val <= 47) || (0 <= j_val && j_val <= 3) || (60 <= j_val && j_val <= 63))
            color <= 1;
          else if(11 <= i_val && i_val <= 22 && (23 <= j_val && j_val <= 26 || 39 <= j_val && j_val <= 42))
            color <= 1;
          else if(23 <= i_val && i_val <= 26 && 27 <= j_val && j_val <= 38)
            color <= 1;
          else if(27 <= i_val && i_val <= 38 && 39 <= j_val && j_val <= 42)
            color <= 1;
          else
            color <= 2;
        end
        default: begin 
				if(block_type[4] == 0)
					color <= (0 <= i_val && i_val <= 3) || (44 <= i_val && i_val <= 47) || (0 <= j_val && j_val <= 3) || (60 <= j_val && j_val <= 63) ? 1 : 0; 
				else begin
					color <= (0 <= i_val && i_val <= 3) || (44 <= i_val && i_val <= 47) || (0 <= j_val && j_val <= 3) || (60 <= j_val && j_val <= 63) ? 1 : 2; 
				end
			end
		endcase
    end

    case(color) 
      2'b00: begin
        if (x == col_coor && y == row_coor) begin
          red <= 4'b0111;
          green <= 4'b0111;
          blue <= 4'b0000;
        end else begin
          red <= 4'b0000;
          green <= 4'b0000;
          blue <= 4'b0000;
        end
      end
      2'b01: begin
        red <= 4'b1111;
        green <= 4'b1111;
        blue <= 4'b1111;
      end
      default: begin
        if (x == col_coor && y == row_coor) begin
          red <= 4'b0111;
          green <= 4'b0111;
          blue <= 4'b0000;
        end else begin
          red <= 4'b0101;
          green <= 4'b0101;
          blue <= 4'b0101;
        end
      end
    endcase
  end

endmodule
